#include "wxJalakFrame.h"
#include "Images.xpm"
#include <wx/splitter.h>
#include <wx/textfile.h>
#include <wx/artprov.h>

BEGIN_EVENT_TABLE(wxJalakFrame, wxFrame)
    EVT_TOOL  (wxID_OPEN, wxJalakFrame::OnOpenClick)
    EVT_TOOL  (wxID_SAVE, wxJalakFrame::OnSaveClick)
    EVT_TOOL  (kID_MIN, wxJalakFrame::OnMinClick)
    EVT_TOOL  (kID_MID, wxJalakFrame::OnMidClick)
    EVT_TOOL  (kID_MAX, wxJalakFrame::OnMaxClick)
    EVT_TOOL  (kID_RAND, wxJalakFrame::OnRandClick)
    EVT_SCROLL(wxJalakFrame::OnSlide)
END_EVENT_TABLE()

wxJalakFrame::wxJalakFrame(wxApp* owner)
            : wxFrame(NULL, wxID_ANY, wxT("Jalak Editor"), wxDefaultPosition,
                      wxSize(800, 680)), fOwner(owner) {
    // Layout
    wxSplitterWindow* sp1 = new wxSplitterWindow(this, wxID_ANY, wxDefaultPosition,
                                wxDefaultSize, wxSP_3D | wxSP_LIVE_UPDATE);

    // Editor
    wxPanel* _editPnl = new wxPanel(sp1, 0, 0, -1, -1);
    wxFlexGridSizer* sz1 = new wxFlexGridSizer(25, 2, 4, 4);
    for (int i=0; i<25; i++) {
        sz1->Add(new wxStaticText(_editPnl, wxID_ANY,
                 wxString::Format(wxT("Column (%d, %d): "), (i/5)+1, (i%5)+1)));
        fHeightSliders[i] = new wxSlider(_editPnl, wxID_ANY, 0, 0, 19,
                                         wxDefaultPosition, wxSize(120, 20));
        sz1->Add(fHeightSliders[i]);
    }
    _editPnl->SetSizerAndFit(sz1);
    
    // Render window
    fViewer = new wxJalakViewer(sp1, wxID_ANY, wxDefaultPosition, wxSize(200, -1));
    fViewer->SetBackgroundColour(GetBackgroundColour());
    
    sp1->SplitVertically(_editPnl, fViewer, 100);
    
    // Toolbar
    //wxSystemOptions::SetOption(wxT("msw.remap"), 0);
    fToolBar = new wxToolBar(this, wxID_ANY, wxDefaultPosition, wxDefaultSize,
                             wxTB_HORIZONTAL | wxNO_BORDER | wxTB_FLAT);
    fToolBar->AddTool(wxID_OPEN, wxT("Open"), wxArtProvider::GetBitmap(wxART_FILE_OPEN),
                      wxNullBitmap, wxITEM_NORMAL, wxT("Open"),
                      wxT("Open a Height Definition file"));
    fToolBar->AddTool(wxID_SAVE, wxT("Save"), wxArtProvider::GetBitmap(wxART_FILE_SAVE),
                      wxNullBitmap, wxITEM_NORMAL, wxT("Save"),
                      wxT("Save the current configuration to a file"));
    fToolBar->AddSeparator();
    fToolBar->AddTool(kID_MAX, wxT("Max"), wxBitmap(XPM_max),
                      wxNullBitmap, wxITEM_NORMAL, wxT("Max Heights"),
                      wxT("Set columns to max height"));
    fToolBar->AddTool(kID_MID, wxT("Med"), wxBitmap(XPM_med),
                      wxNullBitmap, wxITEM_NORMAL, wxT("Mid Heights"),
                      wxT("Set columns to medium height"));
    fToolBar->AddTool(kID_MIN, wxT("Min"), wxBitmap(XPM_min),
                      wxNullBitmap, wxITEM_NORMAL, wxT("Min Heights"),
                      wxT("Set columns to minimum height"));
    fToolBar->AddTool(kID_RAND, wxT("Random"), wxBitmap(XPM_rand),
                      wxNullBitmap, wxITEM_NORMAL, wxT("Randomize Heights"),
                      wxT("Randomize column heights"));
    fToolBar->Realize();
    SetToolBar(fToolBar);
    
    // Status Bar
    fStatBar = new wxStatusBar(this, wxID_ANY, wxST_SIZEGRIP);
    SetStatusBar(fStatBar);
	fStatBar->SetStatusText(wxT("JalakEdit 1.0 by Michael Hansen"), 0);
    
    // Command-line loading
    srand(time(NULL));
    if (owner->argc > 1 && wxFileExists(owner->argv[1]))
        LoadFile(owner->argv[1]);
    SetupSliders();
}

wxJalakFrame::~wxJalakFrame() { }

void wxJalakFrame::LoadFile(const wxChar* filename) {
    wxTextFile F;
    if (!F.Open(filename)) {
        wxMessageBox(
            wxString::Format(wxT("Error opening %s for reading."), filename),
            wxT("Error"), wxOK | wxICON_ERROR);
        return;
    }
    if (F.GetLineCount() != 25) {
        wxMessageBox(
            wxString::Format(wxT("%s is not a valid Jalak Height file."), filename),
            wxT("Error"), wxOK | wxICON_ERROR);
        return;
    }
    for (int i=0; i<5; i++) {
        for (int j=0; j<5; j++) {
            long field = 0;
            if (!F[(i*5)+j].ToLong(&field) || field < 0 || field > 19) {
                wxMessageBox(
                    wxString::Format(wxT("%s is not a valid Jalak Height file."), filename),
                    wxT("Error"), wxOK | wxICON_ERROR);
                return;
            }
            fViewer->Set(i, j, field);
        }
    }
    F.Close();
    SetupSliders();
    Refresh();
}

void wxJalakFrame::SaveFile(const wxChar* filename) {
    wxTextFile F;
    if (wxFileExists(filename))
        wxRemoveFile(filename);
    if (!F.Create(filename)) {
        wxMessageBox(
            wxString::Format(wxT("Error opening %s for writing."), filename),
            wxT("Error"), wxOK | wxICON_ERROR);
        return;
    }
    for (int i=0; i<5; i++)
        for (int j=0; j<5; j++)
            F.AddLine(wxString::Format(wxT("%d"), fViewer->Get(i, j)), wxTextFileType_Dos);
    F.Write();
    F.Close();
}

void wxJalakFrame::OnOpenClick(wxCommandEvent& evt) {
    wxFileDialog fd(this, wxT("Open Height File"), wxEmptyString,
                    wxEmptyString, wxT("Height Files|*.txt"),
                    wxOPEN | wxFILE_MUST_EXIST);
    if (fd.ShowModal() == wxID_OK)
        LoadFile(fd.GetPath());
}

void wxJalakFrame::OnSaveClick(wxCommandEvent& evt) {
    wxFileDialog fd(this, wxT("Save Height File"), wxEmptyString,
                    wxEmptyString, wxT("Height Files|*.txt"),
                    wxSAVE | wxOVERWRITE_PROMPT);
    if (fd.ShowModal() == wxID_OK)
        SaveFile(fd.GetPath());
}

void wxJalakFrame::OnMinClick(wxCommandEvent& evt) {
    fViewer->SetMinHeights();
    SetupSliders();
}

void wxJalakFrame::OnMidClick(wxCommandEvent& evt) {
    fViewer->SetMidHeights();
    SetupSliders();
}

void wxJalakFrame::OnMaxClick(wxCommandEvent& evt) {
    fViewer->SetMaxHeights();
    SetupSliders();
}

void wxJalakFrame::OnRandClick(wxCommandEvent& evt) {
    fViewer->SetRandHeights();
    SetupSliders();
}

void wxJalakFrame::OnSlide(wxScrollEvent& evt) {
    for (int i=0; i<5; i++)
        for (int j=0; j<5; j++)
            fViewer->Set(i, j, fHeightSliders[(i*5)+j]->GetValue());
    fViewer->Render();
}

void wxJalakFrame::SetupSliders() {
    for (int i=0; i<5; i++)
        for (int j=0; j<5; j++)
            fHeightSliders[(i*5)+j]->SetValue(fViewer->Get(i, j));
}
