#include "wxJalakViewer.h"
#include <GL/gl.h>
#include <GL/glu.h>

BEGIN_EVENT_TABLE(wxJalakViewer, wxGLCanvas)
    EVT_SIZE  (wxJalakViewer::OnSize)
    EVT_PAINT (wxJalakViewer::OnPaint)
    EVT_ERASE_BACKGROUND(wxJalakViewer::OnEraseBackground)
    
    EVT_MOUSE_EVENTS(wxJalakViewer::OnMouse)
END_EVENT_TABLE()

wxJalakViewer::wxJalakViewer(wxWindow* parent, wxWindowID id, const wxPoint& pos,
                             const wxSize& size, long style, const wxString& name)
             : wxGLCanvas(parent, id, pos, size, style | wxFULL_REPAINT_ON_RESIZE, name),
               fInited(false), fViewX(0.0f), fViewZ(0.0f) {
    SetMidHeights();
    fMouseFrom = wxDefaultPosition;
}

wxJalakViewer::~wxJalakViewer() { }

void wxJalakViewer::Render() {
    SetCurrent();

    if (!fInited) {
        glShadeModel(GL_SMOOTH);
        glClearDepth(1.0f);
        glEnable(GL_DEPTH_TEST);
        glDepthFunc(GL_LEQUAL);
        glHint(GL_PERSPECTIVE_CORRECTION_HINT, GL_NICEST);
        fInited = true;
    }
    
    glMatrixMode(GL_MODELVIEW);
    glClearColor(GetBackgroundColour().Red() / 255.0,
                 GetBackgroundColour().Green() / 255.0,
                 GetBackgroundColour().Blue() / 255.0, 1.0f);
    glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
    
    for (int i=0; i<5; i++)
        for (int j=0; j<5; j++)
            DrawBar(i, j);
    
    SwapBuffers();
}

void wxJalakViewer::SetMinHeights() {
    for (int i=0; i<5; i++)
        for (int j=0; j<5; j++)
            fHeights[i][j] = 0;
    Render();
}

void wxJalakViewer::SetMidHeights() {
    for (int i=0; i<5; i++)
        for (int j=0; j<5; j++)
            fHeights[i][j] = 9;
    Render();
}

void wxJalakViewer::SetMaxHeights() {
    for (int i=0; i<5; i++)
        for (int j=0; j<5; j++)
            fHeights[i][j] = 19;
    Render();
}

void wxJalakViewer::SetRandHeights() {
    for (int i=0; i<5; i++)
        for (int j=0; j<5; j++)
            fHeights[i][j] = rand() % 20;
    Render();
}

jheight_t wxJalakViewer::Get(int x, int y) const {
    return fHeights[x][y];
}

void wxJalakViewer::Set(int x, int y, jheight_t height) {
    fHeights[x][y] = height;
}

void wxJalakViewer::OnPaint(wxPaintEvent& evt) {
    wxPaintDC dc(this);
    Render();
}

void wxJalakViewer::OnEraseBackground(wxEraseEvent& evt) { }

void wxJalakViewer::OnSize(wxSizeEvent& evt) {
    wxGLCanvas::OnSize(evt);
    
    int width, height;
    GetClientSize(&width, &height);
    
    SetCurrent();
    glViewport(0, 0, width, height);
    glMatrixMode(GL_PROJECTION);
    glLoadIdentity();
    gluPerspective(45.0f, (float)width/(float)height, 0.1f, 100.0f);
}

void wxJalakViewer::OnMouse(wxMouseEvent& evt) {
    if (evt.ButtonDown(wxMOUSE_BTN_LEFT)) {
        fMouseFrom = evt.GetPosition();
    } else if (evt.ButtonUp(wxMOUSE_BTN_LEFT)) {
        fMouseFrom = wxDefaultPosition;
    } else if (evt.Dragging() && evt.ButtonIsDown(wxMOUSE_BTN_LEFT) &&
               fMouseFrom != wxDefaultPosition) {
        fViewZ += (evt.GetPosition().x - fMouseFrom.x) / 4.0f;
        fViewX -= (fMouseFrom.y - evt.GetPosition().y) / 2.0f;
        if (fViewX > 0.0f) fViewX = 0.0f;
        if (fViewX < -90.0f) fViewX = -90.0f;
    } else {
        evt.Skip();
        return;
    }
    fMouseFrom = evt.GetPosition();
    Render();
    
    // Do hit-testing?
}

void wxJalakViewer::DrawBar(int y, int x) {
    float height = -20.0f + (fHeights[y][x] + 1) * 1.0f;
    glLoadIdentity();
    glTranslatef(0.0f, 0.0f, -40.0f);
    glRotatef(fViewX, 1.0f, 0.0f, 0.0f);
    glRotatef(fViewZ, 0.0f, 0.0f, 1.0f);
    glTranslatef((x * 4.0f) - 10.0f, ((4-y) * 4.0f) - 10.0f, 10.0f);
    
    // Draw outline
    glColor3f(0.0f, 0.0f, 0.0f);
    glBegin(GL_LINES);
      // Top
      glVertex3f(0.09f, 0.09f, height+0.01f);
      glVertex3f(0.09f, 3.91f, height+0.01f);
      glVertex3f(0.09f, 3.91f, height+0.01f);
      glVertex3f(3.91f, 3.91f, height+0.01f);
      glVertex3f(3.91f, 3.91f, height+0.01f);
      glVertex3f(3.91f, 0.09f, height+0.01f);
      glVertex3f(3.91f, 0.09f, height+0.01f);
      glVertex3f(0.09f, 0.09f, height+0.01f);
      
      // Sides
      glVertex3f(0.09f, 0.09f, -20.0f);
      glVertex3f(0.09f, 0.09f, height);
      glVertex3f(0.09f, 3.91f, -20.0f);
      glVertex3f(0.09f, 3.91f, height);
      glVertex3f(3.91f, 0.09f, -20.0f);
      glVertex3f(3.91f, 0.09f, height);
      glVertex3f(3.91f, 3.91f, -20.0f);
      glVertex3f(3.91f, 3.91f, height);
    glEnd();
    
    // Set the corner colors to match those in the game
    float red = 0.25f * (x + y);
    if (red >= 1.0)
        red -= 2.0 * (red - 1.0);
    float green = (0.25f * (4-x));
    float blue = 0.0625f * x * y;
    glColor3f(red * 0.65f, green * 0.7f, blue * 0.7f);
    
    glBegin(GL_QUADS);
      // Left
      glVertex3f(0.1f, 0.1f, -20.0f);
      glVertex3f(0.1f, 3.9f, -20.0f);
      glVertex3f(0.1f, 3.9f, height);
      glVertex3f(0.1f, 0.1f, height);
      
      // Back
      glVertex3f(0.1f, 0.1f, -20.0f);
      glVertex3f(3.9f, 0.1f, -20.0f);
      glVertex3f(3.9f, 0.1f, height);
      glVertex3f(0.1f, 0.1f, height);
      
      // Right
      glVertex3f(3.9f, 0.1f, -20.0f);
      glVertex3f(3.9f, 3.9f, -20.0f);
      glVertex3f(3.9f, 3.9f, height);
      glVertex3f(3.9f, 0.1f, height);
      
      // Front
      glVertex3f(0.1f, 3.9f, -20.0f);
      glVertex3f(3.9f, 3.9f, -20.0f);
      glVertex3f(3.9f, 3.9f, height);
      glVertex3f(0.1f, 3.9f, height);
      
      // Top
      glVertex3f(0.1f, 0.1f, height);
      glVertex3f(0.1f, 3.9f, height);
      glVertex3f(3.9f, 3.9f, height);
      glVertex3f(3.9f, 0.1f, height);
    glEnd();
}
